(document:surround "/std/base")
(document:insert "/std/functions")

(define *iface* (make-cell ""))
(define *networks* (make-cell '()))

(define (get-static-lists)
  (woo-catch/message (thunk
    (let ((iface (cell-ref *iface*)))
      (auth_type-id    enumref `("/net-wifi/auth_types" iface ,iface))
      (pairwise-id     enumref `("/net-wifi/pairwise_types" iface ,iface))))))

(define (get-scan-results)
  (woo-catch/message (thunk
    (let ((iface (cell-ref *iface*)))
      (woo-write "/net-wifi" 'iface iface 'btn_rescan "." )
      (scan-results-id enumref `("/net-wifi/scan_results" iface ,iface))))))

(define (clip n l h)
  (if (< n l) l (if (> n h) h n)))

(define (get-networks)
  (let ((iface (cell-ref *iface*)))
    (woo-catch/message (thunk
      (cell-set! *networks*  (woo-list "/net-wifi/networks" 'iface iface))
      (let ((n (networks-id current)))
        (networks-id     enumref `("/net-wifi/networks" iface ,iface))
        (networks-id current (clip n 0 (networks-id count)) ))))))

(define (current-name) (num2name (networks-id current)))

(define (num2name n)
  (if (< n 0) ""
    (if (>= n (length (cell-ref *networks*))) ""
      (woo-get-option
        (list-ref (cell-ref *networks*) n) 'name))))

(define (name2num name)
  (let find-name ((i 0))
    (if (string= name (num2name i))
      i
      (if (>= i (length (cell-ref *networks*)))
        -1
        (find-name (+ i 1))))))


(define (reset-from-backend)
  (woo-catch/message (thunk
    (let* ( (iface   (cell-ref *iface*))
            (data    (woo-read-first "/net-wifi" 'iface iface 'name (current-name))))

        (iface-label-id  text (string-append "<b>" iface "</b>"))
        (status-label-id text (woo-get-option data 'iface_status))

        (enable-cb-id value (woo-get-option data 'enabled))
        (essid-id     value (woo-get-option data 'essid ""))
        (auth_type-id value (woo-get-option data 'auth_type))
        (key-id value "")
        (pairwise-id  value (woo-get-option data 'pairwise ""))
        (networks-id current (name2num (woo-get-option data 'name "")))
        (auth-type-deactivate)))))

(define (write-to-backend)
  (woo-catch/message
    (thunk (woo-write "/net-wifi"
      'btn_apply "."
      'iface   (cell-ref *iface*)
      'enabled (enable-cb-id value)
      'essid   (essid-id value)
      'auth_type (auth_type-id value)
      'key       (key-id value)
      'pairwise  (pairwise-id  value)
      'name      (current-name))
    (get-networks)
    (reset-from-backend))))

(define (new-network)
  (woo-catch/message (thunk
    (woo-write "/net-wifi" 'iface (cell-ref *iface*) 'btn_new "." 'scanned_net (scan-results-id value))
    (get-networks)
    (reset-from-backend))))

(define (delete-network)
  (woo-catch/message (thunk
    (woo-write "/net-wifi" 'iface (cell-ref *iface*) 'name (current-name) 'btn_delete "." )
    (get-networks)
    (reset-from-backend) )))

(define (auth-type-deactivate)
  (if (auth_type-id value)
    (if (string=? (auth_type-id value) "nopasswd")
      ((widgets key-label-id key-id pairwise-label-id pairwise-id) activity #f)
      (begin
        ((widgets key-label-id key-id) activity #t)
        (if (string= (auth_type-id value) "wep")
          ((widgets pairwise-label-id pairwise-id) activity #f)
          ((widgets pairwise-label-id pairwise-id) activity #t))))))


;;; widgets!

(gridbox columns "0;100"
  (label (_ "Interface:")) (document:id iface-label-id  (label ""))
  (label (_ "Status:"))    (document:id status-label-id (label "" text-wrap #t))
(spacer)
(button align "right" text (_ "Reconfigure interface") (when clicked
  (let ((iface (cell-ref *iface*)))
    (woo-catch/message (thunk
      (woo-write "/net-wifi" 'iface iface 'btn_reconfigure "." )))
    (get-networks)
    (reset-from-backend)
  ))))


(groupbox (_ "Configured networks")

  (gridbox columns "30;70"

    (document:id networks-id (listbox
      (when selected (reset-from-backend) )))

    (gridbox columns "0;100"
      (spacer) (document:id enable-cb-id (checkbox (_ "Enable")))
      (label (_ "Network name:"))    (document:id essid-id (edit ""))
      (label (_ "Auth type:") name "auth_type")       (document:id auth_type-id (combobox name "auth_type" (when selected (auth-type-deactivate))))

      (document:id key-label-id      (label (_ "Change password:") name "key")) (document:id key-id (edit "" name "key"))
      (document:id pairwise-label-id (label (_ "Encryption:") name "pairwise"))      (document:id pairwise-id (combobox name "pairwise"))
      (spacer)
      (gridbox columns "100;0;0;0" (spacer)
        (button text (_ "Apply") (when clicked (write-to-backend)))
        (button text (_ "Reset") (when clicked (reset-from-backend)))
        (button text (_ "Delete") (when clicked (delete-network)))))
  ))

(label (string-append "<b>" (_ "Configure new network") "</b>"))

(gridbox columns "0;100;0;0"
  (label (_ "Networks found:"))
  (document:id scan-results-id (combobox))
  (button text (_ "Rescan") (when clicked (get-scan-results)))
  (button text (_ "Configure") (when clicked (new-network))))

(button (_ "OK")
  align "left"
  (when clicked
    (woo-try "monitor" "/net-wifi" 'mode "off")
    (remove-mailbox message-handler)
    (document:end)))

;;;


(define (message-handler args)
  (and (eq? (car args) 'wpa-supplicant)
    (woo-catch/message (thunk
      (woo-write "/net-wifi" 'iface (cell-ref *iface*) 'btn_status ".")
      (get-networks)
      (status-label-id text
        (woo-get-option (woo-read-first "/net-wifi" 'iface (cell-ref *iface*)) 'iface_status))))))

(document:root (when loaded
  (cell-set! *iface* (global 'interface))
  (get-static-lists)
  (get-scan-results)
  (get-networks)
  (reset-from-backend)
  (woo-try "monitor" "/net-wifi" 'mode "on")
  (add-mailbox message-handler)
  (init-effect)
))

(effect-disable "key"       "auth_type"  "nopasswd")
(effect-disable "pairwise"  "auth_type"  "nopasswd")
(effect-disable "pairwise"  "auth_type"  "wep")

